/*
Module : RESSWITCH.CPP
Purpose: Provides the main app functionality for the ResSwitch program
Created: PJN / 25-09-1997

Copyright (c) 1996 - 2008 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code. 

*/


/////////////////////////////////  Includes  //////////////////////////////////

#include "stdafx.h"
#include "ResSwitch.h"
#include "Videomod.h"
#include "ParametersPromptDlg.h"


///////////////////////////////// Macros / Defines ////////////////////////////

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

#ifndef DISP_CHANGE_BADDUALVIEW
#define DISP_CHANGE_BADDUALVIEW -6
#endif

//The one and only CWinApp derived instance
CResSwitchApp theApp;


//////////////////////////////// Implementation ///////////////////////////////

class CMyCommandLineInfo : public CCommandLineInfo
{
public:
//Constructors / Destructors
  CMyCommandLineInfo();

//Methods
  virtual void ParseParam(LPCTSTR lpszParam, BOOL bFlag, BOOL bLast);

//Member variables
  int     m_nHeight;
  int     m_nWidth;
  int     m_nBitsPerPixel;
  int     m_nFrequency;
  CString m_sDeviceName;
  LONG    m_nPositionX;
  LONG    m_nPositionY;    
  BOOL    m_bDetach;
  BOOL    m_bSetPrimary;
  BOOL    m_bGlobal;
  BOOL    m_bTest;
  BOOL    m_bUpdateRegistry;
  BOOL    m_bNoReset;
  BOOL    m_bReset;
};

CMyCommandLineInfo::CMyCommandLineInfo() : m_nHeight(0),
                                           m_nWidth(0),
                                           m_nBitsPerPixel(0),
                                           m_nFrequency(0),
                                           m_nPositionX(LONG_MAX),
                                           m_nPositionY(LONG_MAX),
                                           m_bDetach(FALSE),
                                           m_bSetPrimary(FALSE),
                                           m_bGlobal(FALSE),
                                           m_bTest(FALSE),
                                           m_bUpdateRegistry(FALSE),
                                           m_bNoReset(FALSE),
                                           m_bReset(FALSE)
{
}

void CMyCommandLineInfo::ParseParam(LPCTSTR lpszParam, BOOL bFlag, BOOL bLast)
{
  if (bFlag)
  {
    CString sParam(lpszParam);
    CString sParamUpper(sParam);
    sParamUpper.MakeUpper();
    
    //Parse out the output filename if any
    if (sParamUpper.Find(_T("WIDTH:")) == 0)
      m_nWidth = _ttoi(sParam.Right(sParam.GetLength() - 6));
    else if (sParamUpper.Find(_T("HEIGHT:")) == 0)
      m_nHeight = _ttoi(sParam.Right(sParam.GetLength() - 7));
    else if (sParamUpper.Find(_T("FREQ:")) == 0)
      m_nFrequency = _ttoi(sParam.Right(sParam.GetLength() - 5));
    else if (sParamUpper.Find(_T("DEPTH:")) == 0)
      m_nBitsPerPixel = _ttoi(sParam.Right(sParam.GetLength() - 6));
    else if (sParamUpper.Find(_T("POSITIONX:")) == 0)
      m_nPositionX = _ttoi(sParam.Right(sParam.GetLength() - 10));
    else if (sParamUpper.Find(_T("POSITIONY:")) == 0)
      m_nPositionY = _ttoi(sParam.Right(sParam.GetLength() - 10));
    else if (sParamUpper.Find(_T("DEVICE:")) == 0)
      m_sDeviceName = sParam.Right(sParam.GetLength() - 7);
    else if (sParamUpper == _T("DETACH"))
      m_bDetach = TRUE;
    else if (sParamUpper == _T("PRIMARY"))
      m_bSetPrimary = TRUE;
    else if (sParamUpper == _T("GLOBAL"))
      m_bGlobal = TRUE;
    else if (sParamUpper == _T("TEST"))
      m_bTest = TRUE;
    else if (sParamUpper == _T("UPDATEREGISTRY"))
      m_bUpdateRegistry = TRUE;
    else if (sParamUpper == _T("NORESET"))
      m_bNoReset = TRUE;
    else if (sParamUpper == _T("RESET"))
      m_bReset = TRUE;
    else
    {
      //Let the parent class do its stuff
      CCommandLineInfo::ParseParam(lpszParam, bFlag, bLast);
    }
  }
  else
  {
    //Let the parent class do its stuff
    CCommandLineInfo::ParseParam(lpszParam, bFlag, bLast);
  }
}


BEGIN_MESSAGE_MAP(CResSwitchApp, CWinApp)
END_MESSAGE_MAP()

CResSwitchApp::CResSwitchApp()
{
}

BOOL CResSwitchApp::InitInstance()
{
	//Parse the command line for our parameters
	CMyCommandLineInfo cmdInfo;
	ParseCommandLine(cmdInfo);
  if (__argc < 2)
	{
    //Display an error message if we failed to parse the command line
    CParametersPromptDlg dlg;
    dlg.DoModal();
    return FALSE;
  }

  //Initialize the DEVMOCE structure
  DEVMODE dm;
  memset(&dm, 0, sizeof(dm));
  dm.dmSize = sizeof(dm);
  if (cmdInfo.m_nBitsPerPixel)
  {
    dm.dmFields |= DM_BITSPERPEL;
    dm.dmBitsPerPel = cmdInfo.m_nBitsPerPixel;
  }
  if (cmdInfo.m_nWidth)
  {
    dm.dmFields |= DM_PELSWIDTH;
    dm.dmPelsWidth = cmdInfo.m_nWidth;
  }
  if (cmdInfo.m_nHeight)
  {
    dm.dmFields |= DM_PELSHEIGHT;
    dm.dmPelsHeight = cmdInfo.m_nHeight;
  }
  if (cmdInfo.m_nFrequency)
  {
    dm.dmFields |= DM_DISPLAYFREQUENCY;
    dm.dmDisplayFrequency = cmdInfo.m_nFrequency;
  }
  if (cmdInfo.m_nPositionX != LONG_MAX && cmdInfo.m_nPositionY != LONG_MAX)
  {
    dm.dmFields |= DM_POSITION;
    dm.dmPosition.x = cmdInfo.m_nPositionX;
    dm.dmPosition.y = cmdInfo.m_nPositionY;
  }

  //Call the function to do the work
  DWORD dwFlags = 0;
  if (cmdInfo.m_bGlobal)
    dwFlags |= CDS_GLOBAL;
  if (cmdInfo.m_bDetach)
  {
    dm.dmFields |= DM_POSITION;
    dm.dmPelsWidth = 0;
    dm.dmPelsHeight = 0;
  }
  if (cmdInfo.m_bTest)
    dwFlags |= CDS_TEST;
  if (cmdInfo.m_bSetPrimary)
    dwFlags |= CDS_SET_PRIMARY;
  if (cmdInfo.m_bUpdateRegistry)
    dwFlags |= CDS_UPDATEREGISTRY;
  if (cmdInfo.m_bNoReset)
    dwFlags |= CDS_NORESET;
  if (cmdInfo.m_bReset)
    dwFlags |= CDS_RESET;

  LONG nResult = ChangeDisplaySettingsEx(cmdInfo.m_sDeviceName.GetLength() ? cmdInfo.m_sDeviceName.operator LPCTSTR() : NULL, &dm, NULL, dwFlags, NULL);

  //Handle the error
  switch (nResult)
  {
    case DISP_CHANGE_SUCCESSFUL:
    {
      break; //Nothing to do since the call was successful
    }
    case DISP_CHANGE_BADDUALVIEW:
    {
      AfxMessageBox(_T("The settings change was unsuccessful because the system is dualview capable"), MB_OK | MB_ICONSTOP);
      break;
    }
    case DISP_CHANGE_BADFLAGS:
    {
      AfxMessageBox(_T("An invalid set of flags was provided"), MB_OK | MB_ICONSTOP);
      break;
    }
    case DISP_CHANGE_BADMODE:
    {
      AfxMessageBox(_T("The graphics mode is not supported"), MB_OK | MB_ICONSTOP);
      break;
    }
    case DISP_CHANGE_BADPARAM:
    {
      AfxMessageBox(_T("An invalid parameter was provided"), MB_OK | MB_ICONSTOP);
      break;
    }
    case DISP_CHANGE_FAILED:
    {
      AfxMessageBox(_T("The display driver failed the specified graphics mode"), MB_OK | MB_ICONSTOP);
      break;
    }
    case DISP_CHANGE_NOTUPDATED:
    {
      AfxMessageBox(_T("Unable to write settings to the registry"), MB_OK | MB_ICONSTOP);
      break;
    }
    case DISP_CHANGE_RESTART:
    {
      AfxMessageBox(_T("The computer must be restarted for the graphics mode to work"), MB_OK | MB_ICONSTOP);
      break;
    }
    default:
    {
      CString sError;
      sError.Format(_T("An unexpected error occured calling the Windows function \"ChangeDisplaySettingsEx\", Error:%d"), nResult);
      AfxMessageBox(sError, MB_OK | MB_ICONSTOP);
      break;
    }
  }

	return FALSE;
}
