/*
Module : VIDEOMOD.H
Purpose: Defines the interface to some handy classes which wrap access 
         to the EnumDisplayDevices and EnumDisplaySettings SDK functions

Copyright (c) 1999 - 2008 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code. 

*/


///////////////////// Macros / Structs etc ////////////////////////////////////

#pragma once

#ifndef __VIDEOMOD_H__
#define __VIDEOMOD_H__

#ifndef __AFXTEMPL_H__
#pragma message("To avoid this message, please put afxtempl.h in your pre compiled header (normally stdafx.h)")
#include <afxtempl.h>
#endif


/////////////////////////// Classes ///////////////////////////////////////////

//class wrapper for a display device
class CDisplayDevice
{
public:
//Constructors / Destructors
  CDisplayDevice() : m_dwStateFlags(0)
  {
  };

//Public variables
  CString m_sDeviceName;
  CString m_sDeviceString;
  DWORD   m_dwStateFlags;
  CString m_sDeviceID;
};

//definition of the array which contains display devices
typedef CArray<CDisplayDevice, CDisplayDevice&> CDisplayDevices;

//Class wrapper for a displa mode
class CDisplayMode
{
public:
//Constructors / Destructors
  CDisplayMode() : m_dwBitsPerPixel(0),
                 m_dwWidth(0),
                 m_dwHeight(0),
                 m_dwFrequency(0),
                 m_dwDisplayFlags(0)
  {
  }

//Public variables
  DWORD m_dwBitsPerPixel;
  DWORD m_dwWidth;
  DWORD m_dwHeight;
  DWORD m_dwFrequency;
  DWORD m_dwDisplayFlags;
};

//Defintion of the array which contains display modes
typedef CArray<CDisplayMode, CDisplayMode&> CDisplayModes; 

//Class which enumerates all the display devices
class CAvailableDisplayDevices
{
public:
  static BOOL Get(CDisplayDevices& devices);
};

//Class which enumerates the display modes for a particular display device
class CAvailableDisplayModes
{
public:
  static BOOL GetCurrentForPrimaryDisplay(CDisplayMode& mode);
  static BOOL GetAvailable(LPCTSTR pszDeviceName, CDisplayModes& modes);
};

#endif //__VIDEOMOD_H__
